# Encoder for OpenLaserTag IR transmitter using synchronous code
# on Sony SIRC protocol.
# Tomas Krejci [Njord]

from micropython import const
from olt_lib.ir_tx import IR


# Bit reverse a 32 bit value
def rbit32(v):
    v = (v & 0x0000FFFF) << 16 | (v & 0xFFFF0000) >> 16
    v = (v & 0x00FF00FF) << 8 | (v & 0xFF00FF00) >> 8
    v = (v & 0x0F0F0F0F) << 4 | (v & 0xF0F0F0F0) >> 4
    v = (v & 0x33333333) << 2 | (v & 0xCCCCCCCC) >> 2
    return (v & 0x55555555) << 1 | (v & 0xAAAAAAAA) >> 1


class LT_ABC(IR):
    def __init__(self, pin, bits, freq, verbose):
        super().__init__(pin, freq, 3 + bits * 2, 40, verbose)  # 30 -> 40
        if bits != 24:
            raise ValueError("OLT only support 24 bits.")
        # if bits not in (20, 24):
        #    raise ValueError('bits must be 20 or 24.')
        self.bits = bits

    async def tx(self, tx1, tx2, tx3):
        bits = self.bits
        v = tx3 & 0xFF
        v |= (tx2 & 0xFF) << 8
        v |= (tx1 & 0xFF) << 16

        v = rbit32(v)
        v = v >> 8

        self.append(2400, 600)
        for _ in range(bits):
            self.append(1200 if v & 1 else 600, 600)
            v >>= 1


# OLT specifies 56KHz
class LT_24(LT_ABC):
    valid = (0xFF, 0xFF, 0xFF)  # Max tx1, tx2, tx3

    def __init__(self, pin, freq=56000, verbose=False):
        super().__init__(pin, 24, freq, verbose)
